@@ -1,15 +1,15 @@
-// Copyright (c) 2019 ETH Zurich, Mattia Gollub (mattia.gollub@bsse.ethz.ch)
+// Copyright (c) 2021 ETH Zurich, Mattia Gollub (mattia.gollub@bsse.ethz.ch)
 // Computational Systems Biology group, D-BSSE
 //
 // This software is freely available under the GNU General Public License v3.
 // See the LICENSE file or http://www.gnu.org/licenses/ for further information.
 
-#ifndef HYPERFLUX_FLUX_CONSTRAINTS_H
-#define HYPERFLUX_FLUX_CONSTRAINTS_H
+#ifndef PTA_FLUX_CONSTRAINTS_H
+#define PTA_FLUX_CONSTRAINTS_H
 
 #include <Eigen/Dense>
 
-namespace hyperflux {
+namespace pta {
 
 struct FluxConstraints {
     Eigen::MatrixXd S;
@@ -17,6 +17,6 @@ struct FluxConstraints {
     Eigen::VectorXd upper_bounds;
 };
 
-}  // namespace hyperflux
+} // namespace pta
 
 #endif
\ No newline at end of file
