@@ -1,15 +1,15 @@
-// Copyright (c) 2019 ETH Zurich, Mattia Gollub (mattia.gollub@bsse.ethz.ch)
+// Copyright (c) 2021 ETH Zurich, Mattia Gollub (mattia.gollub@bsse.ethz.ch)
 // Computational Systems Biology group, D-BSSE
 //
 // This software is freely available under the GNU General Public License v3.
 // See the LICENSE file or http://www.gnu.org/licenses/ for further information.
 
-#ifndef HYPERFLUX_FREE_ENERGY_SAMPLING_SETTINGS_H
-#define HYPERFLUX_FREE_ENERGY_SAMPLING_SETTINGS_H
+#ifndef PTA_FREE_ENERGY_SAMPLING_SETTINGS_H
+#define PTA_FREE_ENERGY_SAMPLING_SETTINGS_H
 
 #include <settings.h>
 
-namespace hyperflux {
+namespace pta {
 
 struct FreeEnergySamplingSettings : public samply::Settings {
     FreeEnergySamplingSettings() = default;
@@ -17,8 +17,7 @@ struct FreeEnergySamplingSettings : public samply::Settings {
 
     FreeEnergySamplingSettings(const samply::Settings base_settings)
         : samply::Settings(base_settings)
-    {
-    }
+    {}
 
     // Numerics and performance settings.
     double truncation_multiplier = 1.0;
@@ -32,6 +31,6 @@ struct FreeEnergySamplingSettings : public samply::Settings {
     size_t steps_thinning_directions;
 };
 
-}  // namespace hyperflux
+} // namespace pta
 
 #endif
\ No newline at end of file
