@@ -1,16 +1,18 @@
-// Copyright (c) 2019 ETH Zurich, Mattia Gollub (mattia.gollub@bsse.ethz.ch)
+// Copyright (c) 2021 ETH Zurich, Mattia Gollub (mattia.gollub@bsse.ethz.ch)
 // Computational Systems Biology group, D-BSSE
 //
 // This software is freely available under the GNU General Public License v3.
 // See the LICENSE file or http://www.gnu.org/licenses/ for further information.
 
-#ifndef HYPERFLUX_UTILITIES_H
-#define HYPERFLUX_UTILITIES_H
+#ifndef PTA_UTILITIES_H
+#define PTA_UTILITIES_H
 
+#include "settings.h"
 #include <algorithm>
+#include <vector>
 
-namespace hyperflux {
-namespace utilities {
+namespace pta {
+namespace utils {
 
 template <class InputIt, class UnaryPredicate>
 constexpr InputIt find_if_precedent(InputIt first, InputIt last, UnaryPredicate p)
@@ -35,7 +37,29 @@ inline ForwardIt remove_if_precedent(ForwardIt first, ForwardIt last, UnaryPredi
     return first;
 }
 
-}  // namespace utilities
-}  // namespace hyperflux
+inline std::vector<size_t>
+get_chains_per_thread(const size_t num_chains, const size_t num_threads)
+{
+    const size_t min_chains_per_thread = num_chains / num_threads;
+    const size_t reminder = num_chains - min_chains_per_thread * num_threads;
+
+    if (min_chains_per_thread == 0u) {
+        return std::vector<size_t>(reminder, 1u);
+    }
+    else {
+        std::vector<size_t> result(num_threads, min_chains_per_thread);
+        for (size_t i = 0u; i < reminder; ++i)
+            result[i]++;
+        return result;
+    }
+}
+
+inline size_t estimate_num_states(const samply::Settings& settings)
+{
+    return (settings.num_steps - settings.num_skipped_steps) / settings.steps_thinning;
+}
+
+} // namespace utils
+} // namespace pta
 
 #endif
\ No newline at end of file
