@@ -22,5 +22,6 @@ void add_common_python_bindings(py::module& m)
         .def_readwrite("steps_thinning", &Settings::steps_thinning)
         .def_readwrite("num_skipped_steps", &Settings::num_skipped_steps)
         .def_readwrite("log_interval", &Settings::console_logging_interval_ms)
-        .def_readwrite("log_directory", &Settings::log_directory);
+        .def_readwrite("log_directory", &Settings::log_directory)
+        .def_readwrite("max_threads", &Settings::max_threads);;
 }
\ No newline at end of file
