@@ -4,8 +4,13 @@
 // This software is freely available under the GNU General Public License v3.
 // See the LICENSE file or http://www.gnu.org/licenses/ for further information.
 
+#ifndef PTA_COMMON_PYTHON_BINDINGS
+#define PTA_COMMON_PYTHON_BINDINGS
+
 #include <pybind11/pybind11.h>
 
 namespace py = pybind11;
 
-void add_common_python_bindings(py::module& m);
\ No newline at end of file
+void add_common_python_bindings(py::module& m);
+
+#endif
\ No newline at end of file
