@@ -11,14 +11,12 @@
 #include <pybind11/stl.h>
 
 #include "common_python_bindings.h"
-#include "pta_python_bindings.h"
+#include "tfs_python_bindings.h"
 #include "us_python_bindings.h"
 
 #define STRINGIFY(x) #x
 #define MACRO_STRINGIFY(x) STRINGIFY(x)
 
-namespace py = pybind11;
-
 //==============================================================================
 //  Define Pybind11 bindings.
 //==============================================================================
@@ -31,7 +29,7 @@ PYBIND11_MODULE(_pta_python_binaries, m)
 
     add_common_python_bindings(m);
     add_us_python_bindings(m);
-    add_pta_python_bindings(m);
+    add_tfs_python_bindings(m);
 
 #ifdef VERSION_INFO
     m.attr("__version__") = MACRO_STRINGIFY(VERSION_INFO);
