@@ -13,6 +13,7 @@
 #include <pybind11/eigen.h>
 #include <pybind11/iostream.h>
 #include <pybind11/pybind11.h>
+#include <thread>
 
 #include "direction_samplers/coordinate_direction_sampler.h"
 #include "python_helper.h"
@@ -67,6 +68,7 @@ py::array_t<double> sample_flux_space_uniform(
 
     // Create and initialize a sampler for each thread.
     std::vector<Sampler> samplers;
+    samplers.reserve(chains_per_thread.size());
     size_t state_idx = 0u;
     for (size_t thread_idx = 0; thread_idx < chains_per_thread.size(); ++thread_idx) {
         // Create the loggers, binding the state logger to the common storage.
@@ -87,6 +89,7 @@ py::array_t<double> sample_flux_space_uniform(
 
         // Next start the simulations.
         std::vector<std::thread> threads;
+        threads.reserve(chains_per_thread.size());
         for (auto& sampler : samplers) {
             threads.emplace_back(
                 &Sampler::simulate<Sampler::DurationFormat>, &sampler,
