@@ -7,6 +7,7 @@
 #ifndef SAMPLY_UNIFORM_PDF_SAMPLER_H
 #define SAMPLY_UNIFORM_PDF_SAMPLER_H
 
+#include "commons.h"
 #include "helpers/sampling_helper.h"
 #include "intersections_packet.h"
 #include "rays_packet.h"
@@ -19,9 +20,8 @@ namespace samply {
  *
  * @tparam ScalarType Scalar type for sampling operations.
  */
-template <typename ScalarType>
-class UniformPdfSampler {
-public:
+template <typename ScalarType> class UniformPdfSampler {
+  public:
     /**
      * @brief Scalar type used for sampling operations.
      */
@@ -46,9 +46,11 @@ public:
      * @return Vector containing the sampled points on each ray.
      */
     template <typename RaysPacketType>
-    Vector<Scalar> sample1d(const RaysPacketType& rays,
-                            const IntersectionsPacket<Scalar>& intersections,
-                            SamplingHelper& sampling_helper);
+    void sample1d(
+        Vector<ScalarType>& result_t,
+        const RaysPacketType& rays,
+        const IntersectionsPacket<Scalar>& intersections,
+        SamplingHelper& sampling_helper);
 
     /**
      * @brief Get a reparametrized version of this object such that sampling
@@ -57,11 +59,14 @@ public:
      * @return Reparametrized version of this object, containing the
      * transforms mapping between the two parametrizations.
      */
-    ReparametrizedObject<UniformPdfSampler> get_optimally_reparametrized_sampler()
-        const;
+    ReparametrizedObject<UniformPdfSampler>
+    get_optimally_reparametrized_sampler() const;
 
-private:
+  private:
     Eigen::Index num_dimensions_;
+
+    // Persistent temporary variables, to avoid reallocation at each call.
+    Vector<Scalar> weights_;
 };
 
 //==============================================================================
@@ -71,32 +76,37 @@ private:
 template <typename ScalarType>
 UniformPdfSampler<ScalarType>::UniformPdfSampler(const Eigen::Index num_dimensions)
     : num_dimensions_(num_dimensions)
-{
-}
+{}
 
 template <typename ScalarType>
 template <typename RaysPacketType>
-inline Vector<ScalarType> UniformPdfSampler<ScalarType>::sample1d(
+inline void UniformPdfSampler<ScalarType>::sample1d(
+    Vector<ScalarType>& result_t,
     const RaysPacketType& rays,
     const IntersectionsPacket<ScalarType>& intersections,
     SamplingHelper& sampling_helper)
 {
-    Vector<Scalar> t;
     if (intersections.one_ray_one_intersection) {
         // If each ray has exactly one segment we sample on those segments
         // directly.
-        t = sampling_helper.get_random_uniform(intersections.segment_starts,
-                                               intersections.segment_ends);
-    } else {
+        result_t = sampling_helper.get_random_uniform(
+            intersections.segment_starts, intersections.segment_ends);
+    }
+    else {
         // Otherwise, for each ray we need to first sample one out of the
         // segments belonging to the ray and then sample a point from it.
         auto sample_on_segments_union = [&](const auto& segment_starts,
                                             const auto& segment_ends,
                                             const Eigen::Index ray_index) -> Scalar {
             // Choose the segment from which the next point must be sampled.
-            auto segment_lengths = segment_ends - segment_starts;
+            const Eigen::Index num_segments = segment_starts.size();
+            if (weights_.size() < num_segments)
+                weights_.resize(num_segments);
+            auto segment_weights = weights_.head(num_segments);
+
+            segment_weights = segment_ends - segment_starts;
             Eigen::Index sampled_segment_idx =
-                sampling_helper.sample_index_with_weights<Scalar>(segment_lengths);
+                sampling_helper.sample_index_with_weights<Scalar>(segment_weights);
 
             // Sample the next point uniformly on the chosen segment.
             return sampling_helper.get_random_uniform_scalar(
@@ -104,9 +114,9 @@ inline Vector<ScalarType> UniformPdfSampler<ScalarType>::sample1d(
         };
 
         // Apply the sampling function to each ray.
-        t = intersections.for_each_ray(sample_on_segments_union);
+        result_t.resize(intersections.num_rays);
+        intersections.for_each_ray(result_t, sample_on_segments_union);
     }
-    return t;
 }
 
 template <typename ScalarType>
@@ -118,6 +128,6 @@ UniformPdfSampler<ScalarType>::get_optimally_reparametrized_sampler() const
         UniformPdfSampler<Scalar>(num_dimensions_));
 }
 
-}  // namespace samply
+} // namespace samply
 
 #endif
\ No newline at end of file
