@@ -20,13 +20,14 @@ namespace samply {
  * @tparam Logger Type of the logger monitoring the state of the simulation.
  * @tparam StatesAllocator Type of the allocator for containers of states.
  */
-template <typename Derived,
-          typename State_,
-          typename InternalState_,
-          template <typename, typename>
-          typename Logger>
+template <
+    typename Derived,
+    typename State_,
+    typename InternalState_,
+    template <typename, typename>
+    typename Logger>
 class MarkovChain {
-public:
+  public:
     /**
      * @brief Duration type used for computing elapsed times.
      */
@@ -39,13 +40,12 @@ public:
     typedef InternalState_ InternalState;
 
     class LazyChainStateRef {
-    public:
+      public:
         LazyChainStateRef(Derived& markov_chain, const InternalState& internal_state)
-            : markov_chain_(markov_chain),
-              internal_state_(internal_state),
-              has_state_(false)
-        {
-        }
+            : markov_chain_(markov_chain)
+            , internal_state_(internal_state)
+            , has_state_(false)
+        {}
 
         const State& get()
         {
@@ -56,12 +56,9 @@ public:
             return state_;
         }
 
-        const InternalState& get_internal_state() const
-        {
-            return internal_state_;
-        }
+        const InternalState& get_internal_state() const { return internal_state_; }
 
-    private:
+      private:
         Derived& markov_chain_;
         const InternalState& internal_state_;
         State state_;
@@ -95,8 +92,8 @@ public:
      * @return Number of stored states at the end of the simulation.
      */
     template <typename InputDuration = DurationFormat>
-    size_t simulate(const size_t num_steps,
-                    const InputDuration max_time = DurationFormat::max());
+    size_t simulate(
+        const size_t num_steps, const InputDuration max_time = DurationFormat::max());
 
     /**
      * @brief Set the state of the chain. If the chain already contains states,
@@ -112,7 +109,7 @@ public:
 
     LoggerType& get_logger();
 
-protected:
+  protected:
     virtual void initialize(const State& state) = 0;
     /**
      * @brief Execute one step of the simulation.
@@ -120,11 +117,11 @@ protected:
      * @param state Current state of the chain.
      * @return The new state of the chain.
      */
-    virtual State next_state(const State& state) = 0;
+    virtual void simulate_step(State& state) = 0;
     virtual State convert_from_internal_state(const InternalState& internal_state) = 0;
     virtual InternalState convert_to_internal_state(const State& state) = 0;
 
-private:
+  private:
     // Vector containing all the stored states of the chain.
     InternalState internal_state_;
 
@@ -136,9 +133,9 @@ private:
 //	MarkovChain public methods implementation.
 //==============================================================================
 
-#define _SAMPLY_MARKOVCHAIN_TEMPLATE_DECL                     \
-    typename Derived, typename State, typename InternalState, \
-        template <typename, typename>                         \
+#define _SAMPLY_MARKOVCHAIN_TEMPLATE_DECL                                              \
+    typename Derived, typename State, typename InternalState,                          \
+        template <typename, typename>                                                  \
         typename Logger
 #define _SAMPLY_MARKOVCHAIN_TEMPLATE_SPEC Derived, State, InternalState, Logger
 
@@ -146,8 +143,7 @@ template <_SAMPLY_MARKOVCHAIN_TEMPLATE_DECL>
 inline MarkovChain<_SAMPLY_MARKOVCHAIN_TEMPLATE_SPEC>::MarkovChain(
     const LoggerType& logger)
     : logger_(logger)
-{
-}
+{}
 
 template <_SAMPLY_MARKOVCHAIN_TEMPLATE_DECL>
 template <typename InputDuration>
@@ -155,7 +151,7 @@ inline size_t MarkovChain<_SAMPLY_MARKOVCHAIN_TEMPLATE_SPEC>::simulate(
     const size_t num_steps, const InputDuration max_time)
 {
     using std::chrono::duration_cast;
-    using chain_clock = std::chrono::high_resolution_clock;
+    using chain_clock = std::chrono::steady_clock;
 
     // Validate inputs.
     assert(num_steps > 0);
@@ -172,7 +168,7 @@ inline size_t MarkovChain<_SAMPLY_MARKOVCHAIN_TEMPLATE_SPEC>::simulate(
 
     for (; step_idx < num_steps && !time_limit_reached; ++step_idx) {
         // Simulate the next step of the chain.
-        internal_state_ = static_cast<Derived&>(*this).next_state(internal_state_);
+        static_cast<Derived&>(*this).simulate_step(internal_state_);
 
         // Notify the logger that the step has been completed.
         const auto elapsed_time =
@@ -189,8 +185,8 @@ inline size_t MarkovChain<_SAMPLY_MARKOVCHAIN_TEMPLATE_SPEC>::simulate(
 }
 
 template <_SAMPLY_MARKOVCHAIN_TEMPLATE_DECL>
-inline void MarkovChain<_SAMPLY_MARKOVCHAIN_TEMPLATE_SPEC>::set_state(
-    const State& state)
+inline void
+MarkovChain<_SAMPLY_MARKOVCHAIN_TEMPLATE_SPEC>::set_state(const State& state)
 {
     internal_state_ = static_cast<Derived&>(*this).convert_to_internal_state(state);
 }
@@ -208,6 +204,6 @@ MarkovChain<_SAMPLY_MARKOVCHAIN_TEMPLATE_SPEC>::get_logger()
     return logger_;
 }
 
-}  // namespace samply
+} // namespace samply
 
 #endif
