@@ -26,6 +26,9 @@ struct Settings {
     // Logging settings.
     std::chrono::microseconds console_logging_interval_ms;
     std::string log_directory;
+
+    // Threading settings.
+    size_t max_threads = std::numeric_limits<size_t>::max();
 };
 
 }  // namespace samply
