@@ -20,7 +20,7 @@ namespace samply {
 template <typename Scalar, typename Storage = AffineTransformStorage<Scalar>>
 class LinearTransform
     : public AffineTransformBase<LinearTransform<Scalar, Storage>, Storage> {
-public:
+  public:
     using Base = AffineTransformBase<LinearTransform<Scalar, Storage>, Storage>;
     using Base::Base;
 
@@ -32,8 +32,8 @@ public:
      * @return Transform resulting from the concatenation.
      */
     template <typename OtherTransform, typename OtherStorage>
-    auto operator*(
-        const AffineTransformBase<OtherTransform, OtherStorage>& transform) const;
+    auto
+    operator*(const AffineTransformBase<OtherTransform, OtherStorage>& transform) const;
 
     /**
      * @brief Applies this affine transform to a matrix or vector.
@@ -42,7 +42,7 @@ public:
      * @return The transformed matrix.
      */
     template <typename MatrixType>
-    Matrix<Scalar> operator*(const Eigen::MatrixBase<MatrixType>& matrix) const;
+    auto operator*(const Eigen::MatrixBase<MatrixType>& matrix) const;
 
     /**
      * @brief Returns the inverse of this affine transform. If the inverse is
@@ -62,13 +62,14 @@ template <typename OtherTransform, typename OtherStorage>
 inline auto LinearTransform<Scalar, Storage>::operator*(
     const AffineTransformBase<OtherTransform, OtherStorage>& transform) const
 {
-    return AffineDenseTransform<Scalar>(Base::get_linear() * transform.get_linear(),
-                                        Base::get_linear() * transform.get_shift());
+    return AffineDenseTransform<Scalar>(
+        Base::get_linear() * transform.get_linear(),
+        Base::get_linear() * transform.get_shift());
 }
 
 template <typename Scalar, typename Storage>
 template <typename MatrixType>
-inline Matrix<Scalar> LinearTransform<Scalar, Storage>::operator*(
+inline auto LinearTransform<Scalar, Storage>::operator*(
     const Eigen::MatrixBase<MatrixType>& matrix) const
 {
     return Base::get_linear() * matrix;
@@ -85,6 +86,6 @@ inline auto LinearTransform<Scalar, Storage>::inverse() const
     return LinearTransform(inv_T, Eigen::VectorXd::Zero(inv_T.rows()));
 }
 
-}  // namespace samply
+} // namespace samply
 
 #endif
\ No newline at end of file
