@@ -38,4 +38,8 @@ except ModuleNotFoundError as error:
     else:
         raise
 from .thermodynamic_space import FluxSpace, ThermodynamicSpace, ThermodynamicSpaceBasis
-from .utils import enable_all_logging, get_candidate_thermodynamic_constraints
+from .utils import (
+    enable_all_logging,
+    get_candidate_thermodynamic_constraints,
+    load_example_model,
+)
