@@ -3,8 +3,6 @@
 
 import math
 
-import pint
-
 from .commons import Q
 from .distributions import LogNormalDistribution
 
@@ -79,3 +77,8 @@ tfs_default_min_rel_region_length = 1e-6
 default_max_psrf = 1.1
 """Default treshold on the Potential Scale Reduction Factors (PSRFs).
 """
+default_max_threads = 2 ** 10
+
+min_samples_per_chain = 100
+"""Minimum number of samples that must be drawn from each chain in order to obtain a
+meaningful PSRF score."""
