@@ -1,10 +1,10 @@
+"Estimation of Gibbs free energies using eQuilibrator."
+
 import logging
 from typing import Dict, List, Tuple, Union
 
 import numpy as np
 import numpy.linalg as la
-import scipy.io as sio
-from component_contribution.linalg import LINALG
 from equilibrator_api import ComponentContribution, Reaction
 
 from ..commons import Q
@@ -20,6 +20,13 @@ _DEFAULT_INCORRECT_METABOLITES: List[str] = [
     "bigg.metabolite:glycogen",
 ]
 
+# Hard mappings between identifiers. These will be removed once eQuilibrator updates the
+# MetanetX version.
+_HARD_MAPPINGS: Dict[str, str] = {
+    "bigg.metabolite:ppa": "kegg:C00163",
+    "bigg.metabolite:but": "kegg:C00246",
+}
+
 logger = logging.getLogger(__name__)
 
 
@@ -241,16 +248,22 @@ class EquilibratorGibbsEstimator(GibbsEstimatorInterface):
 
         return pH_correction + phi_correction
 
+    def _make_metabolite_key(
+        self,
+        metabolite: Metabolite,
+    ) -> str:
+        if metabolite.key in self.incorrect_metabolites:
+            return "__unreliable__:" + metabolite.key
+        elif metabolite.key in _HARD_MAPPINGS:
+            return _HARD_MAPPINGS[metabolite.key]
+        else:
+            return metabolite.key
+
     def _make_metabolite_keys(
         self,
         metabolites: List[Metabolite],
     ) -> List[str]:
-        return [
-            m.key
-            if m.key not in self.incorrect_metabolites
-            else "__incorrect__:" + m.key
-            for m in metabolites
-        ]
+        return [self._make_metabolite_key(m) for m in metabolites]
 
     def _try_parse_reaction(
         self,
@@ -261,16 +274,19 @@ class EquilibratorGibbsEstimator(GibbsEstimatorInterface):
         Returns none if eQuilibrator cannot provide an estimate for the free energy
         of the reaction.
         """
+        logging_level = logging.getLogger().level
         logging.getLogger().setLevel(logging.ERROR)
         try:
             reaction = self._eq_api.parse_reaction_formula(rxn_string)
         except Exception:
+            logging.getLogger().setLevel(logging_level)
             return None
 
         for cpd, _ in reaction.items():
             if len(cpd.microspecies) == 0:
+                logging.getLogger().setLevel(logging_level)
                 return None
-        logging.getLogger().setLevel(logging.WARNING)
+        logging.getLogger().setLevel(logging_level)
         return reaction
 
     def _try_get_drg0_prime_single(
