@@ -4,10 +4,11 @@
 import logging
 from typing import List, Optional, Union
 
-import cobra
 import numpy as np
 import pandas as pd
-from cobra.flux_analysis.variability import find_blocked_reactions
+
+import cobra
+from cobra.flux_analysis.variability import flux_variability_analysis
 
 from .constants import (
     Q,
@@ -18,7 +19,11 @@ from .constants import (
     non_intracellular_compartment_ids,
 )
 from .pmo import PmoProblem
-from .utils import get_internal_cycles, tighten_model_bounds
+from .utils import (
+    find_blocked_reactions_from_fva_solution,
+    get_internal_cycles,
+    tighten_model_bounds,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -521,12 +526,20 @@ def prepare_for_pta(
         assessment = StructuralAssessment(model, biomass_id, atpm_id)
         assessment.autoresolve(model, default_bound)
 
+    if remove_blocked_reactions or tighten_bounds:
+        fva_result = flux_variability_analysis(
+            model, loopless=prevent_loops, fraction_of_optimum=0.0
+        )
+
     # Remove blocked reactions to make sure that all reactions can have a
     # well-defined direction.
     if remove_blocked_reactions:
-        model.remove_reactions(find_blocked_reactions(model), remove_orphans=True)
+        model.remove_reactions(
+            find_blocked_reactions_from_fva_solution(model, fva_result=fva_result),
+            remove_orphans=True,
+        )
 
     # Restrict reaction bounds using FVA.
     if tighten_bounds:
-        tighten_model_bounds(model, prevent_loops=prevent_loops)
+        tighten_model_bounds(model, prevent_loops=prevent_loops, fva_result=fva_result)
     logger.info("Model preprocessing completed.")
