@@ -1,6 +1,4 @@
-"""Construction and solution of Probabilistic Metabolic Optimization (PMO)
-problems.
-"""
+"""Construction and solution of Probabilistic Metabolic Optimization (PMO) problems."""
 
 import logging
 import multiprocessing
@@ -9,14 +7,18 @@ import pickle
 import tempfile
 from typing import Any, Callable, Dict, Iterable, List, Optional, Tuple, Union
 
+import cobra
 import cvxpy as cp
 import numpy as np
-import numpy.linalg as la
+import psutil
 from scipy.stats.distributions import chi2
 
-import cobra
-
-from .constants import default_confidence_level, default_max_drg, default_min_drg
+from .constants import (
+    default_confidence_level,
+    default_max_drg,
+    default_max_threads,
+    default_min_drg,
+)
 from .flux_space import FluxSpace
 from .thermodynamic_space import ThermodynamicSpace, ThermodynamicSpaceBasis
 
@@ -38,13 +40,9 @@ class PmoProblemPool:
             Arguments to be passed to the constructor of :code:`PmoProblem`
         """
         # Pick the number of processes based on the number of available CPUs.
-        if num_processes is None:
-            num_processes = multiprocessing.cpu_count()
-            if num_processes is None:
-                logger.warning(
-                    "Cannot determine the number of processors available. Assuming 1."
-                )
-                num_processes = 1
+        num_processes = min(
+            num_processes or default_max_threads, psutil.cpu_count(logical=False)
+        )
         assert num_processes >= 1, "At least one process is needed."
 
         # Create a temporary file to copy the initialization arguments to the worker
@@ -63,9 +61,10 @@ class PmoProblemPool:
     def map(
         self, fn: Callable[["PmoProblem", Any], Any], inputs: Iterable[Any]
     ) -> List[Any]:
-        """Executes a function on each input element. This is the same as the regular
-        :code:`map` function, except that it executes in parallel on the available
-        workers.
+        """Execute a function on each input element.
+
+        This is the same as the regular :code:`map` function, except that it executes in
+        parallel on the available workers.
 
         Parameters
         ----------
@@ -83,7 +82,7 @@ class PmoProblemPool:
         return self._pool.map(PmoProblemPool._pool_task, inputs_with_fn)
 
     def close(self):
-        """Waits for all jobs to be done and closes the pool."""
+        """Wait for all jobs to be done and closes the pool."""
         self._pool.close()
         self._pool.join()
         if os.path.isfile(self._args_filename):
@@ -91,19 +90,18 @@ class PmoProblemPool:
 
     @staticmethod
     def _init_process(args_filename):
-        global _pmo_problem
+        global _pmo_problem  # pylint: disable=global-statement
         argv = pickle.load(open(args_filename, "rb"))
         _pmo_problem = PmoProblem(*argv)
 
     @staticmethod
     def _pool_task(v: Tuple[Callable[["PmoProblem", Any], Any], Any]):
-        global _pmo_problem
+        global _pmo_problem  # pylint: disable=global-statement
         return v[0](_pmo_problem, v[1])
 
 
 class PmoProblem:
-    """Construction and solution of a Probabilistic Metabolic Optimization (PMO)
-    problem.
+    """Construction and solution of a PMO problem.
 
     Parameters
     ----------
@@ -132,6 +130,7 @@ class PmoProblem:
         Dictionary specifying additional options for the solver.
     """
 
+    _REFERENCE_COEFFICIENT = 1e2
     _MAX_COEFFICIENT_RANGE = 5e8
     _DEFAULT_SOLVER_OPTIONS = {cp.GUROBI: {"IntFeasTol": 1e-9, "FeasibilityTol": 1e-9}}
 
@@ -145,12 +144,15 @@ class PmoProblem:
         min_drg: float = default_min_drg,
         max_drg: float = default_max_drg,
         solver: Optional[str] = None,
-        solver_options: dict = {},
+        solver_options: dict = None,
     ):
+        # Create the flux space if needed.
         if isinstance(network, FluxSpace):
             self._F = network.copy()
         else:
             self._F = FluxSpace.from_cobrapy_model(network)
+
+        # Create the basis of the thermodynamic space if needed.
         self._T = thermodynamic_space
         if thermodynamic_space_basis is not None:
             self._B = thermodynamic_space_basis
@@ -168,7 +170,7 @@ class PmoProblem:
         ]
 
         # Scale the flux space to make the problem numerically nicer.
-        self._flux_scale = PmoProblem._scale_flux_space(self.F)
+        self._flux_scale = self._scale_flux_space(self.F)
 
         # Store numerical settings.
         self._confidence_level = confidence_level
@@ -177,11 +179,11 @@ class PmoProblem:
 
         # Initialize solver settings.
         self.solver = solver
-        self.solver_options = solver_options
+        self.solver_options = solver_options or {}
 
         # Flux, thermodynamic and direction variables.
         self._vs = cp.Variable((self.F.n_reactions, 1), "fluxes")
-        self._m = cp.Variable((self.B.dimensionality, 1), "m")
+        self._m = cp.Variable((self.B.dimensionality, 1), "thermo_basis")
         if np.all(
             np.logical_or(
                 self.F.lb[self._rxn_idxs_F_to_T] >= 0,
@@ -226,7 +228,7 @@ class PmoProblem:
         return self._max_drg
 
     @property
-    def flux_scale(self) -> np.array:
+    def flux_scale(self) -> np.ndarray:
         """Gets the vector of scaling factors for the scaled fluxes."""
         return self._flux_scale
 
@@ -238,7 +240,9 @@ class PmoProblem:
         return self._objective
 
     @objective.setter
-    def objective(self, value: Callable[["PmoProblem"], cp.problems.objective.Objective]):
+    def objective(
+        self, value: Callable[["PmoProblem"], cp.problems.objective.Objective]
+    ):
         """Sets the function that constructs the PMO objective."""
         self._objective = value
         self._rebuild_cvxpy_problem()
@@ -265,21 +269,21 @@ class PmoProblem:
         return self._vs.value / self.flux_scale
 
     @property
-    def log_c(self) -> np.array:
+    def log_c(self) -> np.ndarray:
         """Gets the predicted log-concentrations. Can be none if concentrations are not
         represented explicitely.
         """
         return self.B.to_log_conc(self.m.value)
 
     @property
-    def drg0(self) -> np.array:
+    def drg0(self) -> np.ndarray:
         """Gets the predicted standard reaction energies. Can be none if standard
         reaction energies are not represented explicitely.
         """
         return self.B.to_drg0(self.m.value)
 
     @property
-    def drg(self) -> np.array:
+    def drg(self) -> np.ndarray:
         """Gets the predicted reaction energies. Can be none if reaction energies are
         not represented explicitely.
         """
@@ -323,17 +327,17 @@ class PmoProblem:
     @property
     def flux_lb_constraint(self) -> cp.constraints.constraint.Constraint:
         """Gets the lower bound constraint for fluxes."""
-        return (self._flux_lb_constraint,)
+        return self._flux_lb_constraint
 
     @property
     def flux_ub_constraint(self) -> cp.constraints.constraint.Constraint:
         """Gets the upper bound constraint for fluxes."""
-        return (self._flux_ub_constraint,)
+        return self._flux_ub_constraint
 
     @property
     def steady_state_constraint(self) -> cp.constraints.constraint.Constraint:
         """Gets the steady state constraint."""
-        return (self._steady_state_constraint,)
+        return self._steady_state_constraint
 
     @property
     def confidence_constraint(self) -> cp.constraints.constraint.Constraint:
@@ -368,15 +372,23 @@ class PmoProblem:
 
         # Solve the optimization problem.
         self._problem.solve(solver=self.solver, verbose=verbose, **options)
+        if self._problem.status in ["optimal", "optimal_inaccurate"]:
+            assert np.all(
+                np.sign(self.d.value - 0.5) != np.sign(self.drg)
+            ), "The directions and the free energies in the solution are inconsistent."
+            assert np.all(
+                np.sign(self.d.value - 0.5) == np.sign(self.v[self._rxn_idxs_F_to_T])
+            ), "The directions and the fluxes in the solution are inconsistent."
+
         return self._problem.status
 
-    def rebuild_for_directions(self, directions: np.array) -> "PmoProblem":
+    def rebuild_for_directions(self, directions: np.ndarray) -> "PmoProblem":
         """Construct a copy of this PMO problem constrained to the given reaction
         directions.
 
         Parameters
         ----------
-        directions : np.array
+        directions : np.ndarray
             Vector containing the directions (0: backward, 1: forward) of the reactions
             in T.
 
@@ -422,8 +434,7 @@ class PmoProblem:
             self.solver_options,
         )
 
-    @staticmethod
-    def _scale_flux_space(F: FluxSpace) -> np.array:
+    def _scale_flux_space(self, F: FluxSpace) -> np.ndarray:
         """Rescale the flux space in order to reduce the range of coefficients in S, lb
         and ub.
         """
@@ -433,7 +444,9 @@ class PmoProblem:
         # Limit the maximum scaling factor, otherwise we may obtain bad range of
         # coefficients in S. This is a heuristic, and we should pick the value
         # with a better method.
-        col_scale = np.minimum(reference / max_absolute_flux, 100)
+        col_scale = np.minimum(
+            reference / max_absolute_flux, self._REFERENCE_COEFFICIENT
+        )
         F.lb = F.lb * col_scale
         F.ub = F.ub * col_scale
         F.S = F.S @ np.diag(1 / col_scale.ravel())
@@ -478,7 +491,7 @@ class PmoProblem:
             logger.warning(
                 "The coefficient range of the flux bounds "
                 f"({np.max(self._big_M_f) / min_flux_bound}) is close or "
-                f"larger than {self.MAX_COEFFICIENT_RANGE} "
+                f"larger than {self._MAX_COEFFICIENT_RANGE} "
                 "and may lead to numerical errors. It is recommended to "
                 "simplify the model to reduce the range."
             )
@@ -492,12 +505,6 @@ class PmoProblem:
             (self.F.n_metabolites, 1)
         )
 
-        # Thermodynamic space constraints.
-        CI_square = chi2.ppf(self._confidence_level, self.B.dimensionality)
-        self._m_constraint = (
-            cp.atoms.quad_form(self.m, np.identity(self.B.dimensionality)) <= CI_square
-        )
-
         # Mappings between fluxes and constrained fluxes.
         v_to_v_con = np.identity(self.F.n_reactions)[self._rxn_idxs_F_to_T, :]
 
@@ -530,9 +537,17 @@ class PmoProblem:
             self._flux_lb_constraint,
             self._flux_ub_constraint,
             self._steady_state_constraint,
-            self._m_constraint,
         ] + self._sign_constraints
 
+        # Thermodynamic space constraints.
+        if self.confidence_level < 1.0:
+            CI_square = chi2.ppf(self._confidence_level, self.B.dimensionality)
+            self._m_constraint = (
+                cp.atoms.quad_form(self.m, np.identity(self.B.dimensionality))
+                <= CI_square
+            )
+            self._constraints += [self._m_constraint]
+
     def _rebuild_cvxpy_problem(self):
         """Rebuild the CVXPY representation of the PMO problem."""
         self._problem = cp.Problem(self.objective(self), self._constraints)
