@@ -1,21 +1,26 @@
+"""Common classes and methods for sampling algorithms."""
+
+import abc
 import datetime
 import math
-import os
 from typing import Any, Callable
 
+import _pta_python_binaries as pb
 import numpy as np
 import pandas as pd
+import psutil
 
-from _pta_python_binaries import SamplerSettings
-
-from ..constants import default_max_psrf, default_min_chains
+from ..constants import (
+    default_max_psrf,
+    default_max_threads,
+    default_min_chains,
+    min_samples_per_chain,
+)
 
 
 class SamplingException(Exception):
     """Raised when an exception occurs during sampling."""
 
-    pass
-
 
 class SamplingResult:
     """Encapsulates the result of a sampler.
@@ -26,17 +31,39 @@ class SamplingResult:
         Data frame containing the samples as rows.
     psrf : pd.Series
         Series containing the PSRF of each variable.
+    basis_samples : pd.DataFrame, optional
+        The samples in the basis.
+    chains : np.ndarray, optional
+        The simulated chains.
     """
 
-    def __init__(self, samples: pd.DataFrame, psrf: pd.Series):
+    def __init__(
+        self,
+        samples: pd.DataFrame,
+        psrf: pd.Series,
+        basis_samples: pd.DataFrame = None,
+        chains: np.ndarray = None,
+    ):
         self._samples = samples
         self._psrf = psrf
+        self._basis_samples = basis_samples
+        self._chains = chains
+
+    @property
+    def basis_samples(self) -> pd.DataFrame:
+        """Gets a data frame containing the samples in the basis."""
+        return self._basis_samples
 
     @property
     def samples(self) -> pd.DataFrame:
         """Gets a data frame containing the samples."""
         return self._samples
 
+    @property
+    def chains(self) -> np.ndarray:
+        """Get the simulated chains."""
+        return self._chains
+
     @property
     def psrf(self) -> pd.Series:
         """Gets the Potential Scale Reduction Factor (PSRF) of each variable."""
@@ -58,19 +85,78 @@ class SamplingResult:
         return np.all(self.psrf <= max_psrf)
 
 
-def sample_from_chains(chains: np.array, num_samples: int) -> np.array:
+class SamplerInterface(metaclass=abc.ABCMeta):
+    """Interface for an MCMC sampler."""
+
+    @property
+    @abc.abstractmethod
+    def dimensionality(self) -> int:
+        """Get the dimensionality of the sampling space."""
+
+    @abc.abstractmethod
+    def simulate(
+        self,
+        settings: pb.SamplerSettings,
+        initial_points: np.ndarray,
+        directions_transform: np.ndarray,
+    ) -> Any:
+        """Run the sampler with the given parameters.
+
+        Parameters
+        ----------
+        settings : pb.SamplerSettings
+            Sampling settings.
+        initial_points : np.ndarray
+            The initial points for the chains.
+        directions_transform : np.ndarray
+            The transform for the directions sampler.
+        """
+
+    @abc.abstractmethod
+    def compute_psrf(self, result: Any) -> pd.Series:
+        """Compute the potential scale reduction factors for the variables of interest
+        on a given set of chains.
+
+        Parameters
+        ----------
+        result : Any
+            The result of the sampling function.
+
+        Returns
+        -------
+        pd.Series
+            The computed potential scale reduction factors.
+        """
+
+    @abc.abstractmethod
+    def get_chains(self, result: Any) -> np.ndarray:
+        """Extract the simulated chains from a given result.
+
+        Parameters
+        ----------
+        result : Any
+            The result of the native sampling function.
+
+        Returns
+        -------
+        np.ndarray
+            The simulated chains.
+        """
+
+
+def sample_from_chains(chains: np.ndarray, num_samples: int) -> np.ndarray:
     """Draws samples from a given set of chains.
 
     Parameters
     ----------
-    chains : np.array
+    chains : np.ndarray
         Numpy 3D array containing the chains.
     num_samples : int
         Number of samples to draw.
 
     Returns
     -------
-    np.array
+    np.ndarray
         Array of samples.
     """
     samples = np.hstack(chains)
@@ -78,18 +164,18 @@ def sample_from_chains(chains: np.array, num_samples: int) -> np.array:
     return samples[:, indices].T
 
 
-def split_chains(chains: np.array) -> np.array:
+def split_chains(chains: np.ndarray) -> np.ndarray:
     """Split the chains in two, threating each half as a new chain. This is often used
     to detect systematic trends in a random walk.
 
     Parameters
     ----------
-    chains : np.array
+    chains : np.ndarray
         Numpy 3D array containing the input chains.
 
     Returns
     -------
-    np.array
+    np.ndarray
         Numpy 3D array containing the resulting chains.
     """
     half_chain_idx = math.floor(chains.shape[0] / 2)
@@ -101,19 +187,21 @@ def split_chains(chains: np.array) -> np.array:
     )
 
 
-def apply_to_chains(chains: np.array, f: Callable[[np.array], np.array]) -> np.array:
+def apply_to_chains(
+    chains: np.ndarray, f: Callable[[np.ndarray], np.ndarray]
+) -> np.ndarray:
     """Apply a function to each sample of a group of chains.
 
     Parameters
     ----------
-    chains : np.array
+    chains : np.ndarray
         Numpy 3D array containing the input chains.
-    f : Callable[[np.array], np.array]
+    f : Callable[[np.ndarray], np.ndarray]
         Function to apply to each sample.
 
     Returns
     -------
-    np.array
+    np.ndarray
         Numpy 3D array containing the transformed samples.
     """
     n_chains = chains.shape[2]
@@ -123,20 +211,21 @@ def apply_to_chains(chains: np.array, f: Callable[[np.array], np.array]) -> np.a
     return np.dstack(transformed_chains)
 
 
-def split_R(chains: np.array) -> np.array:
+def split_R(chains: np.ndarray) -> np.ndarray:
     """Compute the split-R  (or Potential Scale Reduction Factor) of each variable in
     the given chains.
 
     Parameters
     ----------
-    chains : np.array
+    chains : np.ndarray
         Numpy 3D array containing the input chains.
 
     Returns
     -------
-    np.array
+    np.ndarray
         Vector containing the split-R value for each variable.
     """
+    EPSILON = 1e8
     chains = split_chains(chains)
     n_steps, n_params, n_chains = chains.shape
     R = np.empty(n_params)
@@ -156,8 +245,8 @@ def split_R(chains: np.array) -> np.array:
         W = 1 / n_chains * np.sum(sample_variance)
 
         if (
-            np.all(np.abs(sample_variance / chain_means) < 1e-8)
-            and np.abs(B / sample_mean) < 1e-8
+            np.all(np.abs(sample_variance / np.abs(chain_means) + EPSILON) < EPSILON)
+            and np.abs(B / np.abs(sample_mean) + EPSILON) < EPSILON
         ):
             # If the variance-to-mean ratios within each chain and between all chains
             # are very low, we can assume that the variable is practically constant. Set
@@ -174,13 +263,12 @@ def split_R(chains: np.array) -> np.array:
 
 
 def fill_common_sampling_settings(
-    settings: SamplerSettings,
-    log_directory: str,
+    settings: pb.SamplerSettings,
     num_samples: int,
     num_steps: int,
     num_chains: int = -1,
     num_warmup_steps: int = -1,
-    log_interval: datetime.timedelta = None,
+    max_threads: int = default_max_threads,
 ):
     """Fills default values for common sampling settings.
 
@@ -212,11 +300,12 @@ def fill_common_sampling_settings(
     if num_chains == 0:
         raise ValueError("Sampling requires at least one chain")
 
+    settings.max_threads = min(psutil.cpu_count(logical=False), max_threads)
+
     # If the number of chains is not specified, use at least as many as the number of
     # CPUs.
     if num_chains < 0:
-        num_avail_cpus = os.cpu_count() or 1
-        num_chains = max(num_avail_cpus, default_min_chains)
+        num_chains = max(settings.max_threads, default_min_chains)
 
     # If the number of warmup steps is not specified, set it to half the number of
     # steps.
@@ -230,15 +319,13 @@ def fill_common_sampling_settings(
         )
 
     # Compute the steps thinning and adjust the total steps count if needed.
-    samples_per_chain = math.ceil(num_samples / num_chains)
+    samples_per_chain = max(math.ceil(num_samples / num_chains), min_samples_per_chain)
     steps_thinning = math.ceil((num_steps - num_warmup_steps) / samples_per_chain)
     num_steps = num_warmup_steps + samples_per_chain * steps_thinning
 
-    log_interval = log_interval or datetime.timedelta(seconds=1)
-
     settings.num_steps = num_steps
     settings.num_chains = num_chains
     settings.steps_thinning = steps_thinning
     settings.num_skipped_steps = num_warmup_steps
-    settings.log_interval = log_interval
-    settings.log_directory = log_directory
+    settings.log_interval = datetime.timedelta(seconds=1)
+    settings.log_directory = ""
