@@ -5,11 +5,13 @@ import math
 from pathlib import Path
 from typing import List, Optional, Tuple, Union
 
-import cobra
-import efmtool
 import numpy as np
 import numpy.linalg as la
+import pandas as pd
 import pkg_resources
+
+import cobra
+import efmtool
 from cobra.flux_analysis.variability import (
     find_blocked_reactions,
     flux_variability_analysis,
@@ -52,7 +54,7 @@ def floor(value: float, decimals: int = 0) -> float:
     float
         The rounded value.
     """
-    return math.floor(value * (10 ** decimals)) / (10 ** decimals)
+    return math.floor(value * (10**decimals)) / (10**decimals)
 
 
 def ceil(value: float, decimals: int = 0):
@@ -70,7 +72,7 @@ def ceil(value: float, decimals: int = 0):
     float
         The rounded value.
     """
-    return math.ceil(value * (10 ** decimals)) / (10 ** decimals)
+    return math.ceil(value * (10**decimals)) / (10**decimals)
 
 
 def get_internal_reactions(model: cobra.Model) -> List[str]:
@@ -95,6 +97,7 @@ def tighten_model_bounds(
     margin: float = 1e-4,
     round_to_digits: int = 6,
     prevent_loops: bool = False,
+    fva_result: pd.DataFrame = None,
 ):
     """Apply FVA to a model to reduce the range of the flux bounds. The FVA result (plus
     a margin) is applied to each reaction if it is tighter than the bounds in the model.
@@ -102,7 +105,7 @@ def tighten_model_bounds(
     Parameters
     ----------
     model : cobra.Model
-
+        The model to analyze.
     margin : float, optional
         Margin to be applied to the FVA result when the computed bounds are tighter than
         the original ones. This avoids overconstraining the model in case of numerical
@@ -110,26 +113,71 @@ def tighten_model_bounds(
     round_to_digits : int, optional
         Number of digits to which tighter bounds should be rounded to. This limits the
         range of the coefficients for optimization problems. By default 6.
+    prevent_loops : bool, optional
+        If no FVA solution is provided, determines whether regular or loopless FVA will
+        be used.
+    fva_result : pd.DataFrame, optional
+        Precomputed FVA result. If specified, FVA will not be run again, by default
+        None.
     """
-    fva_result = flux_variability_analysis(
-        model, loopless=prevent_loops, fraction_of_optimum=0.0
-    )
+    if fva_result is None:
+        fva_result = flux_variability_analysis(
+            model, loopless=prevent_loops, fraction_of_optimum=0.0
+        )
 
     for r in model.reactions:
         lb = fva_result.loc[r.id, "minimum"]
         ub = fva_result.loc[r.id, "maximum"]
 
-        if lb >= 0 and lb <= margin:
+        if 0 <= lb <= margin:
             r.lower_bound = max(r.lower_bound, 0.0)
         else:
             r.lower_bound = max(r.lower_bound, floor(lb - margin, round_to_digits))
 
-        if ub <= 0 and ub >= -margin:
+        if -margin <= ub <= 0:
             r.upper_bound = min(r.upper_bound, 0.0)
         else:
             r.upper_bound = min(r.upper_bound, ceil(ub + margin, round_to_digits))
 
 
+def find_blocked_reactions_from_fva_solution(
+    model: cobra.Model,
+    prevent_loops: bool = False,
+    zero_cutoff: Optional[float] = None,
+    fva_result: pd.DataFrame = None,
+) -> List[str]:
+    """Similar to cobrapy's find_blocked_reactions(), but optionally takes a precomputed
+    FVA solution as input.
+
+    Parameters
+    ----------
+    model : cobra.Model
+        The model to analyze.
+    prevent_loops : bool, optional
+        If no FVA solution is provided, determines whether regular or loopless FVA will
+        be used.
+    zero_cutoff : Optional[float], optional
+        Flux value which is considered to effectively be zero. The default
+        is set to use `model.tolerance` (default None).
+    fva_result : pd.DataFrame, optional
+        Precomputed FVA result. If specified, FVA will not be run again, by default
+        None.
+
+    Returns
+    -------
+    List[str]
+        List of identifiers of the blocked reactions.
+    """
+    zero_cutoff = cobra.flux_analysis.helpers.normalize_cutoff(model, zero_cutoff)
+
+    if fva_result is None:
+        fva_result = flux_variability_analysis(
+            model, loopless=prevent_loops, fraction_of_optimum=0.0
+        )
+
+    return fva_result[fva_result.abs().max(axis=1) < zero_cutoff].index.tolist()
+
+
 def get_candidate_thermodynamic_constraints(
     model: Union[FluxSpace, cobra.Model],
     metabolite_interpreter: MetaboliteInterpreter = None,
@@ -367,14 +415,15 @@ def get_internal_cycles(
 
 
 def to_reactions_idxs(
-    reactions: Union[List[int], List[str], cobra.DictList], model: cobra.Model
+    reactions: Union[List[int], List[str], cobra.DictList, List[cobra.Reaction]],
+    model: cobra.Model,
 ) -> List[int]:
     """Utility function to obtain a list of reaction indices from different
     representations.
 
     Parameters
     ----------
-    reactions : Union[List[int], List[str], cobra.DictList]
+    reactions : Union[List[int], List[str], cobra.DictList, List[cobra.Reaction]]
         Input list of reactions. Reactions can be defined through their index in the
         model, their identifiers, or with the reactions themselves.
     model : cobra.Model
@@ -396,20 +445,23 @@ def to_reactions_idxs(
         return reactions  # type: ignore
     if isinstance(reactions, list) and isinstance(reactions[0], str):
         return [model.reactions.index(r) for r in reactions]
-    if isinstance(reactions, cobra.DictList):
+    if isinstance(reactions, cobra.DictList) or (
+        isinstance(reactions, list) and isinstance(reactions[0], cobra.Reaction)
+    ):
         return [model.reactions.index(r) for r in reactions]
     raise Exception("Unsupported reaction list format")
 
 
 def to_reactions_ids(
-    reactions: Union[List[int], List[str], cobra.DictList], model: cobra.Model
+    reactions: Union[List[int], List[str], cobra.DictList, List[cobra.Reaction]],
+    model: cobra.Model,
 ) -> List[str]:
     """Utility function to obtain a list of reaction identifiers from different
     representations.
 
     Parameters
     ----------
-    reactions : Union[List[int], List[str], cobra.DictList]
+    reactions : Union[List[int], List[str], cobra.DictList, List[cobra.Reaction]]
         Input list of reactions. Reactions can be defined through their index in the
         model, their identifiers, or with the reactions themselves.
     model : cobra.Model
@@ -431,7 +483,9 @@ def to_reactions_ids(
         return [model.reactions[i].id for i in reactions]
     if isinstance(reactions, list) and isinstance(reactions[0], str):
         return reactions  # type: ignore
-    if isinstance(reactions, cobra.DictList):
+    if isinstance(reactions, cobra.DictList) or (
+        isinstance(reactions, list) and isinstance(reactions[0], cobra.Reaction)
+    ):
         return [r.id for r in reactions]
     raise Exception("Unsupported reaction list format")
 
@@ -614,7 +668,7 @@ def find_rotation_matrix(x: np.ndarray, y: np.ndarray) -> np.ndarray:
     v = v / np.linalg.norm(v)
 
     cos_t = np.dot(x, y) / np.linalg.norm(x) / np.linalg.norm(y)
-    sin_t = np.sqrt(1 - cos_t ** 2)
+    sin_t = np.sqrt(1 - cos_t**2)
 
     return (
         np.identity(len(x))
